@@ -2,17 +2,17 @@
 
 if [ -z "$NVC_INSTALL_DEST" ]; then
   if command -v cygpath &>/dev/null; then
-    export NVC_INSTALL_DEST="$(cygpath -m $HOME)/.nvc/lib"
+    export NVC_INSTALL_DEST="$(cygpath -m "$HOME")/.nvc/lib"
   else
-    export NVC_INSTALL_DEST=$HOME/.nvc/lib
+    export NVC_INSTALL_DEST="$HOME/.nvc/lib"
   fi
 fi
 
-export SCRIPT_DIR=$(dirname $BASH_SOURCE)
+export SCRIPT_DIR="$(dirname "$BASH_SOURCE")"
 
 _safe () {
   echo $*
-  $*
+  "$@"
   [ $? = 0 ] || exit 1
 }
 
@@ -21,7 +21,7 @@ _nvc () {
   local _dest=$NVC_INSTALL_DEST
   local _opts="--std=${STD:-1993} --work=$_dest/$_work -L$_dest $GLOBAL_OPTS"
   [ -d $_dest ] || _safe mkdir -p $_dest
-  _safe ${NVC:-nvc} $_opts $*
+  _safe "${NVC:-nvc}" $_opts $*
 }
 
 analyse () {
