@@ -3,7 +3,7 @@
 # Called by "nvc --install ise".
 #
 
-. $(dirname $BASH_SOURCE)/functions.sh
+. "$(dirname "$BASH_SOURCE")/functions.sh"
 
 if [ -z "$ICECUBE2" ]; then
   cat >&2 <<EOF
@@ -24,20 +24,20 @@ EOF
 fi
 
 if [ -n "$MSYSTEM" ]; then
-  ICECUBE2=$(cygpath -u $ICECUBE2)
+  ICECUBE2="$(cygpath -u "$ICECUBE2")"
 fi
 
 echo "Using iCEcube2 installation in $ICECUBE2"
 echo
 
-src=$ICECUBE2/vhdl
+src="$ICECUBE2/vhdl"
 
 GLOBAL_OPTS="-M 64m"
 A_OPTS="--relaxed"
 
 for STD in ${NVC_STD:-1993 2008}; do
   WORK=ice$(std_suffix $STD)
-  analyse $src/vcomponent_vital.vhd
-  analyse $src/sb_ice_syn_vital.vhd
-  analyse $src/sb_ice_lc_vital.vhd
+  analyse "$src/vcomponent_vital.vhd"
+  analyse "$src/sb_ice_syn_vital.vhd"
+  analyse "$src/sb_ice_lc_vital.vhd"
 done
