@@ -3,7 +3,7 @@
 # Called by "nvc --install libero".
 #
 
-. $(dirname $BASH_SOURCE)/functions.sh
+. "$(dirname "$BASH_SOURCE")/functions.sh"
 
 if [ -z "$LIBERO" ]; then
   cat >&2 <<EOF
@@ -24,18 +24,18 @@ EOF
 fi
 
 if [ -n "$MSYSTEM" ]; then
-  LIBERO=$(cygpath -u $LIBERO)
+  LIBERO="$(cygpath -u "$LIBERO")"
 fi
 
 echo "Using Microchip Libero installation in $LIBERO"
 echo
 
-src=$LIBERO/Designer/lib/
+src="$LIBERO/Designer/lib/"
 
 GLOBAL_OPTS="-M 64m"
 A_OPTS="--relaxed"
 
 for STD in ${NVC_STD:-1993 2008}; do
   WORK=axcelerator$(std_suffix $STD)
-  analyse $src/vtl/95/axcelerator.vhd
+  analyse "$src/vtl/95/axcelerator.vhd"
 done
