@@ -27,17 +27,6 @@ typedef struct _cover_exclude_ctx   cover_exclude_ctx_t;
 typedef struct _cover_rpt_buf       cover_rpt_buf_t;
 typedef struct _cover_spec          cover_spec_t;
 
-typedef A(char*) char_array_t;
-
-struct _cover_spec {
-   char_array_t hier_include;
-   char_array_t hier_exclude;
-   char_array_t block_include;
-   char_array_t block_exclude;
-   char_array_t fsm_type_include;
-   char_array_t fsm_type_exclude;
-};
-
 typedef struct _cover_excl_cmd {
    ident_t              hier;
    loc_t                loc;
@@ -64,7 +53,6 @@ typedef struct _cover_ef {
 struct _cover_data {
    cover_mask_t     mask;
    int              array_limit;
-   int              array_depth;
    int              threshold;
    cover_rpt_buf_t *rpt_buf;
    cover_spec_t    *spec;
@@ -80,13 +68,25 @@ typedef struct {
 } line_range_t;
 
 typedef A(line_range_t) range_array_t;
-typedef A(cover_item_t) cov_item_array_t;
+typedef A(cover_item_t *) cov_item_array_t;
 typedef A(cover_scope_t *) scope_array_t;
 
+typedef enum {
+   CSCOPE_NONE,
+   CSCOPE_INSTANCE,
+   CSCOPE_SUBPROG,
+   CSCOPE_PACKAGE,
+   CSCOPE_PROCESS,
+   CSCOPE_USER,
+   CSCOPE_PROPERTY,
+} cscope_kind_t;
+
 typedef struct _cover_scope {
    ident_t           name;
    ident_t           hier;
+   ident_t           block_name;
    loc_t             loc;
+   cscope_kind_t     kind;
    int               branch_label;
    int               stmt_label;
    int               expression_label;
