@@ -461,7 +461,7 @@ static const struct {
    { "Protected type declarations", { [STD_08] = "5.6.2",
         [STD_02] = "3.5.1" } },
    { "Type conversions", { [STD_93] = "7.3.5", [STD_08] = "9.3.6" } },
-   { "External names", { [STD_08] = "8.7" } },
+   { "External names", { [STD_08] = "8.7", [STD_19] = "8.7" } },
    { "Port clauses", { [STD_08] = "6.5.6.3" } },
    { "Subprogram instantiation declarations", { [STD_08] = "4.4" } },
    { "Incomplete type declarations", { [STD_08] = "5.4.2", [STD_93] = "3.3.1",
@@ -471,7 +471,7 @@ static const struct {
    { "Basic identifiers", { [STD_93] = "13.3.1", [STD_08] = "15.4.2" } },
    { "Concurrent signal assignment statements", { [STD_93] = "9.5",
         [STD_08] = "11.6" } },
-   { "Elaboration of a declaration", { [STD_08] = "14.4.2",
+   { "Elaboration of a declaration", { [STD_08] = "14.4.2", [STD_19] = "14.4.2",
         [STD_93] = "12.3.1" } },
    { "Package bodies", { [STD_08] = "4.8", [STD_93] = "2.6" } },
    { "Interface type declarations", { [STD_08] = "6.5.3",
@@ -481,9 +481,10 @@ static const struct {
         [STD_93] = "8.4.1" } },
    { "Subprogram bodies", { [STD_08] = "4.3", [STD_93] = "2.2" } },
    { "Index contraints and discrete ranges", { [STD_93] = "3.2.1.1",
-        [STD_08] = "5.3.2.2" } },
-   { "Aggregates", { [STD_08] = "9.3.3", [STD_93] = "7.3.2" } },
-   { "Interface object declarations", { [STD_08] = "6.5.2",
+        [STD_08] = "5.3.2.2", [STD_19] = "5.3.2.2" } },
+   { "Aggregates", { [STD_08] = "9.3.3", [STD_93] = "7.3.2",
+      [STD_19] = "9.3.3" } },
+   { "Interface object declarations", { [STD_08] = "6.5.2", [STD_19] = "6.5.2",
         [STD_93] = "4.3.2" } },
    { "Association lists", { [STD_08] = "6.5.7", [STD_93] = "4.3.2.2" } },
    { "Protect tool directives", { [STD_08] = "24.1" } },
