@@ -22,6 +22,7 @@
 #include "ident.h"
 #include "jit/jit-priv.h"
 #include "option.h"
+#include "printf.h"
 #include "thread.h"
 
 #include <assert.h>
@@ -298,6 +299,8 @@ code_cache_t *code_cache_new(void)
    shash_put(s, "memcpy", &memcpy);
    shash_put(s, "memset", &memset);
    shash_put(s, "pow", &pow);
+   shash_put(s, "ldexp", &ldexp);
+   shash_put(s, "exp2", &exp2);
 
 #if defined __APPLE__ && defined ARCH_ARM64
    shash_put(s, "bzero", &bzero);
@@ -586,9 +589,9 @@ void code_blob_finalise(code_blob_t *blob, jit_entry_fn_t *entry)
    freespan->base = aligned;
 
    if (opt_get_verbose(OPT_ASM_VERBOSE, istr(span->name))) {
-      color_printf("\n$bold$$blue$");
+      nvc_printf("\n$bold$$blue$");
       code_disassemble(span, 0, NULL);
-      color_printf("$$\n");
+      nvc_printf("$$\n");
    }
 
    __builtin___clear_cache((char *)span->base, (char *)blob->wptr);
@@ -711,9 +714,6 @@ static void code_blob_print_value(text_buf_t *tb, jit_value_t value)
    case JIT_ADDR_ABS:
       tb_printf(tb, "[#%016"PRIx64"]", value.int64);
       break;
-   case JIT_ADDR_COVER:
-      tb_printf(tb, "@%"PRIi64, value.int64);
-      break;
    case JIT_VALUE_LABEL:
       tb_printf(tb, "%d", value.label);
       break;
@@ -831,9 +831,10 @@ static void arm64_patch_page_base_rel21(uint32_t *patch, void *ptr)
    const intptr_t dst_page = (intptr_t)ptr & ~UINT64_C(0xfff);
    const intptr_t src_page = (intptr_t)patch & ~UINT64_C(0xfff);
    const intptr_t upper21 = (dst_page - src_page) >> 12;
-   assert((upper21 & ~UINT64_C(0x1fffff)) == 0);
-   *(uint32_t *)patch |= (upper21 & 3) << 29;
-   *(uint32_t *)patch |= ((upper21 >> 2) & 0x7ffff) << 5;
+   assert(upper21 >= -(1 << 20) && upper21 < (1 << 20));
+   *patch &= ~((0x3 << 29) | (0x7ffff << 5));
+   *patch |= (upper21 & 3) << 29;
+   *patch |= ((upper21 >> 2) & 0x7ffff) << 5;
 }
 #endif
 
