@@ -544,8 +544,8 @@ void __nvc_do_exit(jit_exit_t which, jit_anchor_t *anchor, jit_scalar_t *args,
 
    case JIT_EXIT_OVERFLOW:
       {
-         int32_t lhs   = args[0].integer;
-         int32_t rhs   = args[1].integer;
+         int64_t lhs   = args[0].integer;
+         int64_t rhs   = args[1].integer;
          tree_t  where = args[2].pointer;
 
          x_overflow(lhs, rhs, where);
@@ -1006,6 +1006,13 @@ void __nvc_do_exit(jit_exit_t which, jit_anchor_t *anchor, jit_scalar_t *args,
       }
       break;
 
+   case JIT_EXIT_GET_COUNTERS:
+      {
+         jit_handle_t handle = args[0].integer;
+         args[0].pointer = x_get_counters(handle);
+      }
+      break;
+
    default:
       fatal_trace("unhandled exit %s", jit_exit_name(which));
    }
@@ -1017,17 +1024,20 @@ DLLEXPORT
 void __nvc_vec4op(jit_vec_op_t op, jit_anchor_t *anchor, jit_scalar_t *args,
                   int size)
 {
-   const int nwords = (size + 63) / 64;
-   assert(size > 64);
-
    jit_thread_local_t *thread = jit_attach_thread(anchor);
 
+   const int nwords = (size + 63) / 64;
+
    switch (op) {
    case JIT_VEC_CASE_EQ:
    case JIT_VEC_CASE_NEQ:
+   case JIT_VEC_LOG_EQ:
+   case JIT_VEC_LOG_NEQ:
    case JIT_VEC_AND1:
    case JIT_VEC_OR1:
       {
+         assert(size > 64);
+
          const uint64_t *aleft  = args[0].pointer;
          const uint64_t *bleft  = args[1].pointer;
          const uint64_t *aright = args[2].pointer;
@@ -1045,6 +1055,16 @@ void __nvc_vec4op(jit_vec_op_t op, jit_anchor_t *anchor, jit_scalar_t *args,
                || memcmp(bleft, bright, nwords * sizeof(uint64_t)) != 0;
             bresult = 0;
             break;
+         case JIT_VEC_LOG_EQ:
+            bresult = vec2_or1(size, bleft) | vec2_or1(size, bright);
+            aresult = (memcmp(aleft, aright, nwords * sizeof(uint64_t)) == 0)
+               | bresult;
+            break;
+         case JIT_VEC_LOG_NEQ:
+            bresult = vec2_or1(size, bleft) | vec2_or1(size, bright);
+            aresult = (memcmp(aleft, aright, nwords * sizeof(uint64_t)) != 0)
+               | bresult;
+            break;
          case JIT_VEC_AND1:
             bresult = vec2_or1(size, bleft);
             aresult = vec2_and1(size, aleft) | bresult;
@@ -1062,40 +1082,78 @@ void __nvc_vec4op(jit_vec_op_t op, jit_anchor_t *anchor, jit_scalar_t *args,
       }
       break;
    case JIT_VEC_ZEXT:
+   case JIT_VEC_SEXT:
       {
+         assert(size > 64);
+
          void *ptr = jit_mspace_alloc(nwords * 2 * sizeof(uint64_t));
          uint64_t *aresult = ptr, *bresult = aresult + nwords;
 
-         if (op == JIT_VEC_ZEXT) {
-            const int arg_size = args[2].integer;
-            if (arg_size <= 64) {
-               aresult[0] = args[0].integer;
-               bresult[0] = args[1].integer;
+         const int arg_size = args[2].integer;
+         const uint64_t msb_mask =
+            arg_size % 64 != 0 ? UINT64_C(1) << ((arg_size % 64) - 1) : 0;
+         const bool sext = op == JIT_VEC_SEXT;
 
-               for (int i = 1; i < nwords; i++)
-                  aresult[i] = bresult[i] = 0;
+         if (arg_size <= 64) {
+            const uint64_t aext =
+               sext && (args[0].integer & msb_mask) ? ~UINT64_C(0) : 0;
+            const uint64_t bext =
+               sext && (args[1].integer & msb_mask) ? ~UINT64_C(0) : 0;
+
+            aresult[0] = args[0].integer;
+            bresult[0] = args[1].integer;
+
+            if (arg_size < 64) {
+               aresult[0] |= aext << arg_size;
+               bresult[0] |= bext << arg_size;
             }
-            else {
-               const uint64_t *asrc = args[0].pointer;
-               const uint64_t *bsrc = args[1].pointer;
 
-               for (int i = 0; i < nwords; i++) {
-                  if (i < (arg_size + 63) / 64) {
-                     aresult[i] = asrc[i];
-                     bresult[i] = bsrc[i];
-                  }
-                  else
-                     aresult[i] = bresult[i] = 0;
+            for (int i = 1; i < nwords; i++) {
+               aresult[i] = aext;
+               bresult[i] = bext;
+            }
+         }
+         else {
+            const uint64_t *asrc = args[0].pointer;
+            const uint64_t *bsrc = args[1].pointer;
+
+            const uint64_t aext =
+               sext && (asrc[arg_size / 64] & msb_mask) ? ~UINT64_C(0) : 0;
+            const uint64_t bext =
+               sext && (bsrc[arg_size / 64] & msb_mask) ? ~UINT64_C(0) : 0;
+
+            const int full_words = arg_size / 64;
+            const int partial_index = (arg_size % 64) != 0 ? full_words : -1;
+
+            for (int i = 0; i < nwords; i++) {
+               if (i < full_words) {
+                  aresult[i] = asrc[i];
+                  bresult[i] = bsrc[i];
+               } else if (i == partial_index) {
+                  const int shift = arg_size - i * 64;
+                  aresult[i] = asrc[i] | (aext << shift);
+                  bresult[i] = bsrc[i] | (bext << shift);
+               } else {
+                  aresult[i] = aext;
+                  bresult[i] = bext;
                }
             }
          }
 
+         if (size % 64 != 0) {
+            const uint64_t mask = ~UINT64_C(0) >> (64 - size % 64);
+            aresult[nwords - 1] &= mask;
+            bresult[nwords - 1] &= mask;
+         }
+
          args[0].pointer = aresult;
          args[1].pointer = bresult;
       }
       break;
    case JIT_VEC_INSERT:
       {
+         assert(size > 64);
+
          void *ptr = jit_mspace_alloc(nwords * 2 * sizeof(uint64_t));
          uint64_t *aresult = ptr, *bresult = aresult + nwords;
 
@@ -1155,16 +1213,27 @@ void __nvc_vec4op(jit_vec_op_t op, jit_anchor_t *anchor, jit_scalar_t *args,
       break;
    default:
       {
-         void *ptr = jit_mspace_alloc(nwords * 2 * sizeof(uint64_t));
-         uint64_t *aresult = ptr, *bresult = aresult + nwords;
+         uint64_t *aresult, *bresult, *a2, *b2;
+         if (size > 64) {
+            void *ptr = jit_mspace_alloc(nwords * 2 * sizeof(uint64_t));
+            aresult = ptr;
+            bresult = aresult + nwords;
 
-         const uint64_t *a1 = args[0].pointer;
-         const uint64_t *b1 = args[1].pointer;
-         const uint64_t *a2 = args[2].pointer;
-         const uint64_t *b2 = args[3].pointer;
+            memcpy(aresult, args[0].pointer, nwords * sizeof(uint64_t));
+            memcpy(bresult, args[1].pointer, nwords * sizeof(uint64_t));
 
-         memcpy(aresult, a1, nwords * sizeof(uint64_t));
-         memcpy(bresult, b1, nwords * sizeof(uint64_t));
+            args[0].pointer = aresult;
+            args[1].pointer = bresult;
+
+            a2 = args[2].pointer;
+            b2 = args[3].pointer;
+         }
+         else {
+            aresult = (uint64_t *)&args[0].integer;
+            bresult = (uint64_t *)&args[1].integer;
+            a2 = (uint64_t *)&args[2].integer;
+            b2 = (uint64_t *)&args[3].integer;
+         }
 
          switch (op) {
          case JIT_VEC_ADD:
@@ -1185,15 +1254,18 @@ void __nvc_vec4op(jit_vec_op_t op, jit_anchor_t *anchor, jit_scalar_t *args,
          case JIT_VEC_SHR:
             vec4_shr(size, aresult, bresult, a2, b2);
             break;
+         case JIT_VEC_ASR:
+            vec4_asr(size, aresult, bresult, a2, b2);
+            break;
          case JIT_VEC_NOT:
             vec4_inv(size, aresult, bresult);
             break;
+         case JIT_VEC_EXP:
+            vec4_exp(size, aresult, bresult, a2, b2);
+            break;
          default:
             should_not_reach_here();
          }
-
-         args[0].pointer = aresult;
-         args[1].pointer = bresult;
       }
       break;
    }
@@ -1202,7 +1274,7 @@ void __nvc_vec4op(jit_vec_op_t op, jit_anchor_t *anchor, jit_scalar_t *args,
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// Entry points from AOT compiled code
+// Entry points from compiled code
 
 DLLEXPORT
 void _debug_out(intptr_t val, int32_t reg)
