@@ -1,5 +1,5 @@
 //
-//  Copyright (C) 2022-2024  Nick Gasson
+//  Copyright (C) 2022-2026  Nick Gasson
 //
 //  This program is free software: you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -184,6 +184,7 @@ typedef enum {
    JIT_EXIT_SCHED_DEPOSIT,
    JIT_EXIT_PUT_DRIVER,
    JIT_EXIT_SCHED_INACTIVE,
+   JIT_EXIT_GET_COUNTERS,
 } jit_exit_t;
 
 typedef uint16_t jit_reg_t;
@@ -197,7 +198,6 @@ typedef enum {
    JIT_ADDR_REG,
    JIT_ADDR_ABS,
    JIT_ADDR_CPOOL,
-   JIT_ADDR_COVER,
    JIT_VALUE_LABEL,
    JIT_VALUE_HANDLE,
    JIT_VALUE_EXIT,
@@ -216,11 +216,15 @@ typedef enum {
    JIT_VEC_MOD,
    JIT_VEC_SHL,
    JIT_VEC_SHR,
+   JIT_VEC_ASR,
    JIT_VEC_CASE_EQ,
    JIT_VEC_CASE_NEQ,
+   JIT_VEC_LOG_EQ,
+   JIT_VEC_LOG_NEQ,
    JIT_VEC_NOT,
    JIT_VEC_AND1,
    JIT_VEC_OR1,
+   JIT_VEC_EXP,
 } jit_vec_op_t;
 
 typedef uint32_t jit_label_t;
@@ -232,7 +236,7 @@ typedef struct {
 } jit_vpos_t;
 
 typedef struct {
-   jit_value_kind_t kind : 8;
+   jit_value_kind_t kind;
    int32_t          disp;
    union {
       jit_reg_t    reg;
@@ -302,15 +306,6 @@ typedef enum {
    JIT_FUNC_ERROR,
 } func_state_t;
 
-typedef enum {
-   RELOC_NULL,
-   RELOC_HANDLE,
-   RELOC_FUNC,
-   RELOC_PRIVDATA,
-   RELOC_COVER,
-   RELOC_PROCESSED,
-} reloc_kind_t;
-
 typedef struct {
    ident_t  name;
    unsigned offset;
@@ -324,7 +319,6 @@ typedef struct _jit_func {
    link_tab_t     *linktab;
    mptr_t          privdata;
    jit_ir_t       *irbuf;
-   int32_t        *counters;
    unsigned char  *cpool;
    unsigned        framesz;
    unsigned        nirs;
@@ -408,7 +402,6 @@ void **jit_get_privdata_ptr(jit_t *j, jit_func_t *f);
 void jit_tier_up(jit_func_t *f);
 jit_thread_local_t *jit_thread_local(void);
 void jit_fill_irbuf(jit_func_t *f);
-int32_t *jit_get_cover_ptr(jit_func_t *f, jit_value_t addr);
 jit_entry_fn_t jit_bind_intrinsic(ident_t name);
 jit_thread_local_t *jit_attach_thread(jit_anchor_t *anchor);
 
