@@ -1281,9 +1281,6 @@ static void jit_x86_get_copy(code_blob_t *blob, x86_operand_t dst,
    case JIT_ADDR_CPOOL:
       MOV(dst, PTR(blob->func->cpool + src.int64), __QWORD);
       break;
-   case JIT_ADDR_COVER:
-      MOV(dst, PTR(jit_get_cover_ptr(blob->func, src)), __QWORD);
-      break;
    default:
       fatal_trace("cannot handle value kind %d in jit_x86_get", src.kind);
    }
@@ -1322,9 +1319,6 @@ static x86_operand_t jit_x86_get(code_blob_t *blob, x86_operand_t tmp,
    case JIT_ADDR_CPOOL:
       MOV(tmp, PTR(blob->func->cpool + src.int64), __QWORD);
       return tmp;
-   case JIT_ADDR_COVER:
-      MOV(tmp, PTR(jit_get_cover_ptr(blob->func, src)), __QWORD);
-      return tmp;
    default:
       fatal_trace("cannot handle value kind %d in jit_x86_get", src.kind);
    }
@@ -1344,9 +1338,6 @@ static x86_operand_t jit_x86_get_addr(code_blob_t *blob, jit_value_t addr,
    case JIT_ADDR_ABS:
       MOV(tmp, IMM(addr.int64), __QWORD);
       return ADDR(tmp, 0);
-   case JIT_ADDR_COVER:
-      MOV(tmp, PTR(jit_get_cover_ptr(blob->func, addr)), __QWORD);
-      return ADDR(tmp, 0);
    default:
       fatal_trace("cannot handle value kind %d in jit_x86_get_addr", addr.kind);
    }
