@@ -52,7 +52,7 @@ typedef struct {
 
 typedef void (*jit_irq_fn_t)(jit_t *, void *);
 
-jit_t *jit_new(unit_registry_t *ur, mir_context_t *mc, cover_data_t *db);
+jit_t *jit_new(unit_registry_t *ur, mir_context_t *mc);
 void jit_free(jit_t *j);
 jit_handle_t jit_compile(jit_t *j, ident_t name);
 jit_handle_t jit_lazy_compile(jit_t *j, ident_t name);
@@ -61,7 +61,6 @@ void *jit_link(jit_t *j, jit_handle_t handle);
 void *jit_get_frame_var(jit_t *j, jit_handle_t handle, ident_t name);
 void jit_set_silent(jit_t *j, bool silent);
 mspace_t *jit_get_mspace(jit_t *j);
-void jit_load_dll(jit_t *j, ident_t name);
 void jit_load_pack(jit_t *j, FILE *f);
 bool jit_exit_status(jit_t *j, int *status);
 void jit_reset_exit_status(jit_t *j);
@@ -69,6 +68,7 @@ void jit_add_tier(jit_t *j, int threshold, const jit_plugin_t *plugin);
 ident_t jit_get_name(jit_t *j, jit_handle_t handle);
 object_t *jit_get_object(jit_t *j, jit_handle_t handle);
 void jit_register_native_plugin(jit_t *j);
+void jit_register_llvm_plugin(jit_t *j);
 void jit_interrupt(jit_t *j, jit_irq_fn_t fn, void *ctx);
 void jit_check_interrupt(jit_t *j);
 void jit_reset(jit_t *j);
