@@ -137,10 +137,9 @@ static void import_link_var(mir_unit_t *mu, mir_import_t *imp, int op)
 
    mir_value_t context = imp->map[vcode_get_arg(op, 0)];
    mir_type_t type = import_type(mu, imp, vtype);
-   ident_t unit_name = vtype_name(vcode_reg_type(vcode_get_arg(op, 0)));
    ident_t name = vcode_get_ident(op);
 
-   imp->map[result] = mir_build_link_var(mu, unit_name, context, name, type);
+   imp->map[result] = mir_build_link_var(mu, context, name, type);
 }
 
 static void import_return(mir_unit_t *mu, mir_import_t *imp, int op)
@@ -750,6 +749,19 @@ static void import_array_ref(mir_unit_t *mu, mir_import_t *imp, int op)
    imp->map[vcode_get_result(op)] = mir_build_array_ref(mu, array, offset);
 }
 
+static void import_table_ref(mir_unit_t *mu, mir_import_t *imp, int op)
+{
+   const int nargs = vcode_count_args(op);
+   assert(nargs >= 2);
+
+   mir_value_t *args LOCAL = xmalloc_array(nargs, sizeof(mir_value_t));
+   for (int i = 0; i < nargs; i++)
+      args[i] = imp->map[vcode_get_arg(op, i)];
+
+   imp->map[vcode_get_result(op)] = mir_build_table_ref(mu, args[0], args[1],
+                                                        args + 2, nargs - 2);
+}
+
 static void import_record_ref(mir_unit_t *mu, mir_import_t *imp, int op)
 {
    mir_value_t record = imp->map[vcode_get_arg(op, 0)];
@@ -946,8 +958,13 @@ static void import_bind_external(mir_unit_t *mu, mir_import_t *imp, int op)
    vcode_type_t vtype = vtype_pointed(vcode_reg_type(result));
    mir_type_t type = import_type(mu, imp, vtype);
 
+   const int nargs = vcode_count_args(op) - 1;
+   mir_value_t *args LOCAL = xmalloc_array(nargs, sizeof(mir_value_t));
+   for (int i = 0; i < nargs; i++)
+      args[i] = imp->map[vcode_get_arg(op, i + 1)];
+
    imp->map[result] = mir_build_bind_external(mu, locus, scope, type,
-                                              MIR_NULL_STAMP);
+                                              MIR_NULL_STAMP, args, nargs);
 }
 
 static void import_unreachable(mir_unit_t *mu, mir_import_t *imp, int op)
@@ -1150,17 +1167,20 @@ static void import_process_init(mir_unit_t *mu, mir_import_t *imp, int op)
 
 static void import_cover_stmt(mir_unit_t *mu, mir_import_t *imp, int op)
 {
-   mir_build_cover_stmt(mu, vcode_get_tag(op));
+   mir_value_t counters = imp->map[vcode_get_arg(op, 0)];
+   mir_build_cover_stmt(mu, counters, vcode_get_tag(op));
 }
 
 static void import_cover_branch(mir_unit_t *mu, mir_import_t *imp, int op)
 {
-   mir_build_cover_branch(mu, vcode_get_tag(op));
+   mir_value_t counters = imp->map[vcode_get_arg(op, 0)];
+   mir_build_cover_branch(mu, counters, vcode_get_tag(op));
 }
 
 static void import_cover_expr(mir_unit_t *mu, mir_import_t *imp, int op)
 {
-   mir_build_cover_expr(mu, vcode_get_tag(op));
+   mir_value_t counters = imp->map[vcode_get_arg(op, 0)];
+   mir_build_cover_expr(mu, counters, vcode_get_tag(op));
 }
 
 static void import_cover_toggle(mir_unit_t *mu, mir_import_t *imp, int op)
@@ -1295,31 +1315,6 @@ static void import_enter_state(mir_unit_t *mu, mir_import_t *imp, int op)
    mir_build_enter_state(mu, state, strong);
 }
 
-static void import_syscall(mir_unit_t *mu, mir_import_t *imp, int op)
-{
-   mir_value_t locus = imp->map[vcode_get_arg(op, 0)];
-   ident_t func = vcode_get_func(op);
-
-   mir_type_t type = MIR_NULL_TYPE;
-   vcode_type_t vtype = vcode_get_type(op);
-   if (vtype != VCODE_INVALID_TYPE)
-      type = import_type(mu, imp, vtype);
-
-   const int nargs = vcode_count_args(op) - 1;
-   mir_value_t *args LOCAL = xmalloc_array(nargs, sizeof(mir_value_t));
-   for (int i = 0; i < nargs; i++)
-      args[i] = imp->map[vcode_get_arg(op, i + 1)];
-
-   vcode_reg_t result = vcode_get_result(op);
-   if (result == VCODE_INVALID_REG)
-      mir_build_syscall(mu, func, type, MIR_NULL_STAMP, locus, args, nargs);
-   else {
-      mir_type_t type = import_type(mu, imp, vcode_reg_type(result));
-      imp->map[result] =  mir_build_syscall(mu, func, type, MIR_NULL_STAMP,
-                                            locus, args, nargs);
-   }
-}
-
 static void import_reflect_value(mir_unit_t *mu, mir_import_t *imp, int op)
 {
    mir_value_t value = imp->map[vcode_get_arg(op, 0)];
@@ -1359,6 +1354,12 @@ static void import_sched_process(mir_unit_t *mu, mir_import_t *imp, int op)
    mir_build_sched_process(mu, delay);
 }
 
+static void import_get_counters(mir_unit_t *mu, mir_import_t *imp, int op)
+{
+   ident_t block = vcode_get_ident(op);
+   imp->map[vcode_get_result(op)] = mir_build_get_counters(mu, block);
+}
+
 static void import_block(mir_unit_t *mu, mir_import_t *imp)
 {
    const int nops = vcode_count_ops();
@@ -1566,6 +1567,9 @@ static void import_block(mir_unit_t *mu, mir_import_t *imp)
       case VCODE_OP_ARRAY_REF:
          import_array_ref(mu, imp, i);
          break;
+      case VCODE_OP_TABLE_REF:
+         import_table_ref(mu, imp, i);
+         break;
       case VCODE_OP_RECORD_REF:
          import_record_ref(mu, imp, i);
          break;
@@ -1751,9 +1755,6 @@ static void import_block(mir_unit_t *mu, mir_import_t *imp)
       case VCODE_OP_ENTER_STATE:
          import_enter_state(mu, imp, i);
          break;
-      case VCODE_OP_SYSCALL:
-         import_syscall(mu, imp, i);
-         break;
       case VCODE_OP_REFLECT_SUBTYPE:
          import_reflect_subtype(mu, imp, i);
          break;
@@ -1766,6 +1767,9 @@ static void import_block(mir_unit_t *mu, mir_import_t *imp)
       case VCODE_OP_SCHED_PROCESS:
          import_sched_process(mu, imp, i);
          break;
+      case VCODE_OP_GET_COUNTERS:
+         import_get_counters(mu, imp, i);
+         break;
       default:
          vcode_dump_with_mark(i, NULL, NULL);
          fatal_trace("cannot import vcode op %s",
@@ -1868,5 +1872,6 @@ mir_unit_t *mir_import(mir_context_t *mc, vcode_unit_t vu)
    free(imp.map);
    free(imp.vars);
 
+   mir_optimise(mu, MIR_PASS_O0);
    return mu;
 }
