@@ -588,6 +588,8 @@ static tree_t simp_attr_ref(tree_t t, simp_ctx_t *ctx)
             else if (dim_i < 1 || dim_i > dimension_of(type))
                break;
          }
+         else if (type_is_generic(type))
+            break;   // Cannot simplify until instantiated
 
          tree_t r = range_of(type, dim_i - 1);
 
@@ -778,12 +780,8 @@ static tree_t simp_process(tree_t t)
       tree_set_loc(p, tree_loc(t));
       tree_set_flag(p, tree_flags(t));
 
-      const int ndecls = tree_decls(t);
-      for (int i = 0; i < ndecls; i++)
-         tree_add_decl(p, tree_decl(t, i));
-
-      for (int i = 0; i < nstmts; i++)
-         tree_add_stmt(p, tree_stmt(t, i));
+      tree_copy_decls(p, t);
+      tree_copy_stmts(p, t);
 
       tree_t w = tree_new(T_WAIT);
       tree_set_ident(w, tree_ident(p));
@@ -887,64 +885,34 @@ static void simp_make_dummy_drivers(tree_t container, tree_list_t *list)
 
 static bool simp_match_case_choice(tree_t alt, int64_t ival)
 {
-   const int nassocs = tree_assocs(alt);
-   for (int j = 0; j < nassocs; j++) {
-      tree_t a = tree_assoc(alt, j);
-      switch (tree_subkind(a)) {
-      case A_NAMED:
-         {
-            int64_t aval;
-            if (folded_int(tree_name(a), &aval) && ival == aval)
-               return true;
-         }
-         break;
-
-      case A_RANGE:
-         {
-            int64_t low, high;
-            if (!folded_bounds(tree_range(a, 0), &low, &high))
-               continue;
-            else if (ival >= low && ival <= high)
-               return true;
-         }
-
-      case A_OTHERS:
-         return true;
+   const int nchoices = tree_choices(alt);
+   for (int j = 0; j < nchoices; j++) {
+      tree_t a = tree_choice(alt, j);
+      if (tree_has_name(a)) {
+         int64_t aval;
+         if (folded_int(tree_name(a), &aval) && ival == aval)
+            return true;
       }
+      else if (tree_ranges(a) > 0) {
+         int64_t low, high;
+         if (!folded_bounds(tree_range(a, 0), &low, &high))
+            continue;
+         else if (ival >= low && ival <= high)
+            return true;
+      }
+      else
+         return true;
    }
 
    return false;
 }
 
-static tree_t simp_case(tree_t t, simp_ctx_t *ctx)
+static tree_t simp_case(tree_t t)
 {
    const int nstmts = tree_stmts(t);
    if (nstmts == 0)
       return NULL;    // All choices are unreachable
 
-   for (int i = 0; i < nstmts; i++) {
-      tree_t alt = tree_stmt(t, i);
-      const int nassocs = tree_assocs(alt);
-      for (int j = 0; j < nassocs; j++) {
-         tree_t a = tree_assoc(alt, j);
-         if (tree_subkind(a) == A_NAMED) {
-            tree_t n = tree_name(a);
-            switch (tree_kind(n)) {
-            case T_LITERAL:
-            case T_STRING:
-            case T_REF:
-               break;
-            default:
-               if (eval_possible(n, ctx->registry, ctx->mir)) {
-                  n = eval_try_fold(ctx->jit, n, ctx->registry, NULL, NULL);
-                  tree_set_name(a, n);
-               }
-               break;
-            }
-         }
-      }
-   }
-
    int64_t ival;
    if (!folded_int(tree_value(t), &ival))
       return t;
@@ -972,9 +940,7 @@ static tree_t simp_case(tree_t t, simp_ctx_t *ctx)
       if (tree_has_ident(t))
          tree_set_ident(seq, tree_ident(t));
 
-      const int nstmts = tree_stmts(alt);
-      for (int i = 0; i < nstmts; i++)
-         tree_add_stmt(seq, tree_stmt(alt, i));
+      tree_copy_stmts(seq, alt);
 
       simp_make_dummy_drivers(seq, &drivers);
       return seq;
@@ -983,6 +949,42 @@ static tree_t simp_case(tree_t t, simp_ctx_t *ctx)
    return NULL;  // No choices can be executed
 }
 
+static tree_t simp_static_expr(tree_t t, simp_ctx_t *ctx)
+{
+   switch (tree_kind(t)) {
+   case T_LITERAL:
+   case T_STRING:
+   case T_REF:
+      return t;
+   default:
+      if (eval_possible(t, ctx->registry, ctx->mir))
+         return eval_try_fold(ctx->jit, t, ctx->registry, NULL, NULL);
+      else
+         return t;
+   }
+}
+
+static tree_t simp_choice(tree_t t, simp_ctx_t *ctx)
+{
+   // Choice names should be static expressions (except in aggregates
+   // with a single element)
+
+   if (tree_has_name(t)) {
+      tree_t n = tree_name(t);
+      tree_set_name(t, simp_static_expr(n, ctx));
+   }
+   else if (tree_ranges(t) > 0) {
+      assert(tree_ranges(t) == 1);
+      tree_t range = tree_range(t, 0);
+      tree_t l = tree_left(range);
+      tree_set_left(range, simp_static_expr(l, ctx));
+      tree_t r = tree_right(range);
+      tree_set_right(range, simp_static_expr(r, ctx));
+   }
+
+   return t;
+}
+
 static tree_t simp_case_generate(tree_t t)
 {
    const int nstmts = tree_stmts(t);
@@ -1010,13 +1012,8 @@ static tree_t simp_case_generate(tree_t t)
       else if (tree_has_ident(t))
          tree_set_ident(seq, tree_ident(t));
 
-      const int ndecls = tree_decls(alt);
-      for (int i = 0; i < ndecls; i++)
-         tree_add_decl(seq, tree_decl(alt, i));
-
-      const int nstmts = tree_stmts(alt);
-      for (int i = 0; i < nstmts; i++)
-         tree_add_stmt(seq, tree_stmt(alt, i));
+      tree_copy_decls(seq, alt);
+      tree_copy_stmts(seq, alt);
 
       return seq;
    }
@@ -1072,10 +1069,7 @@ static tree_t simp_if(tree_t t)
          if (new != NULL && tree_conds(new) > 0) {
             tree_t c2 = tree_new(T_COND_STMT);
             tree_set_loc(c2, tree_loc(c));
-
-            const int nstmts = tree_stmts(c);
-            for (int i = 0; i < nstmts; i++)
-               tree_add_stmt(c2, tree_stmt(c, i));
+            tree_copy_stmts(c2, c);
 
             tree_add_cond(new, c2);
             break;
@@ -1088,9 +1082,7 @@ static tree_t simp_if(tree_t t)
             if (tree_has_ident(t))
                tree_set_ident(b, tree_ident(t));
 
-            const int nstmts = tree_stmts(c);
-            for (int i = 0; i < nstmts; i++)
-               tree_add_stmt(b, tree_stmt(c, i));
+            tree_copy_stmts(b, c);
 
             simp_make_dummy_drivers(b, &drivers);
             return b;
@@ -1262,10 +1254,7 @@ static tree_t simp_select(tree_t t)
    tree_t c = tree_new(kind);
    tree_set_loc(c, tree_loc(t));
    tree_set_value(c, tree_value(t));
-
-   const int nstmts = tree_stmts(t);
-   for (int i = 0; i < nstmts; i++)
-      tree_add_stmt(c, tree_stmt(t, i));
+   tree_copy_stmts(c, t);
 
    if (tree_has_guard(t)) {
       tree_t s0 = tree_stmt(tree_stmt(t, 0), 0);
@@ -1348,13 +1337,8 @@ static tree_t simp_if_generate(tree_t t)
             else if (tree_has_ident(t))
                tree_set_ident(b, tree_ident(t));
 
-            const int ndecls = tree_decls(c);
-            for (int i = 0; i < ndecls; i++)
-               tree_add_decl(b, tree_decl(c, i));
-
-            const int nstmts = tree_stmts(c);
-            for (int i = 0; i < nstmts; i++)
-               tree_add_stmt(b, tree_stmt(c, i));
+            tree_copy_decls(b, c);
+            tree_copy_stmts(b, c);
 
             return b;
          }
@@ -1752,7 +1736,9 @@ static tree_t simp_tree_local(tree_t t, void *_ctx)
    case T_IF:
       return simp_if(t);
    case T_CASE:
-      return simp_case(t, ctx);
+      return simp_case(t);
+   case T_CHOICE:
+      return simp_choice(t, ctx);
    case T_CASE_GENERATE:
       return simp_case_generate(t);
    case T_WHILE:
@@ -1853,7 +1839,7 @@ static tree_t simp_tree_global(tree_t t, void *_ctx)
    case T_IF:
       return simp_if(t);
    case T_CASE:
-      return simp_case(t, ctx);
+      return simp_case(t);
    case T_CASE_GENERATE:
       return simp_case_generate(t);
    case T_WHILE:
