@@ -28,6 +28,7 @@
 #include "rt/structs.h"
 #include "vlog/vlog-node.h"
 #include "vlog/vlog-number.h"
+#include "vlog/vlog-util.h"
 #include "vpi/vpi-macros.h"
 #include "vpi/vpi-model.h"
 #include "vpi/vpi-priv.h"
@@ -156,16 +157,35 @@ typedef struct {
 
 typedef struct {
    c_abstractDecl decl;
+   PLI_UINT32     size;
 } c_net;
 
 DEF_CLASS(net, vpiNet, decl.object);
 
 typedef struct {
    c_abstractDecl decl;
+   PLI_UINT32     size;
+   PLI_UINT32     offset;
 } c_reg;
 
 DEF_CLASS(reg, vpiReg, decl.object);
 
+typedef struct {
+   c_abstractDecl decl;
+   vpiLazyList    elems;
+   c_vpiObject   *left;
+   c_vpiObject   *right;
+} c_regArray;
+
+DEF_CLASS(regArray, vpiRegArray, decl.object);
+
+typedef struct {
+   c_abstractDecl  decl;
+   c_vpiObject    *value;
+} c_parameter;
+
+DEF_CLASS(parameter, vpiParameter, decl.object);
+
 #define HANDLE_BITS      (sizeof(vpiHandle) * 8)
 #define HANDLE_MAX_INDEX ((UINT64_C(1) << (HANDLE_BITS / 2)) - 1)
 
@@ -203,6 +223,7 @@ typedef struct _vpi_context {
 
 static c_vpiObject *build_expr(vlog_node_t v, c_abstractScope *scope);
 static void vpi_lazy_decls(c_vpiObject *obj);
+static void vpi_lazy_elems(c_vpiObject *obj);
 static c_refcounted *is_refcounted(c_vpiObject *obj);
 
 static vpi_context_t *global_context = NULL;   // TODO: thread local
@@ -357,7 +378,9 @@ static c_abstractDecl *is_abstractDecl(c_vpiObject *obj)
 {
    switch (obj->type) {
    case vpiReg:
+   case vpiRegArray:
    case vpiNet:
+   case vpiParameter:
       return container_of(obj, c_abstractDecl, object);
    default:
       return NULL;
@@ -368,12 +391,28 @@ static c_abstractScope *is_abstractScope(c_vpiObject *obj)
 {
    switch (obj->type) {
    case vpiModule:
+   case vpiGenScope:
       return container_of(obj, c_abstractScope, object);
    default:
       return NULL;
    }
 }
 
+static c_constant *is_constantOrParam(c_vpiObject *obj)
+{
+   switch (obj->type) {
+   case vpiConstant:
+      return container_of(obj, c_constant, expr.object);
+   case vpiParameter:
+      {
+         c_vpiObject *value = container_of(obj, c_parameter, decl.object)->value;
+         return is_constantOrParam(value);
+      }
+   default:
+      return NULL;
+   }
+}
+
 static const char *handle_pp(vpiHandle handle)
 {
    static __thread text_buf_t *tb = NULL;
@@ -394,6 +433,10 @@ static const char *handle_pp(vpiHandle handle)
    else {
       c_vpiObject *obj = slot->obj;
       tb_cat(tb, vpi_type_str(obj->type));
+
+      c_operation *op = is_operation(obj);
+      if (op != NULL)
+         tb_printf(tb, " OpType=%s", vpi_op_type_str(op->subtype));
    }
 
    tb_append(tb, '}');
@@ -483,7 +526,7 @@ static void init_tfCall(c_tfCall *call, vlog_node_t v, c_abstractScope *scope)
       vpi_list_add(&(call->args), obj);
 
       c_operation *op = is_operation(obj);
-      if (op != NULL) {
+      if (op != NULL && op->subtype != vpiNullOp) {
          op->argslot = argslot;
          argslot += 3;
       }
@@ -520,16 +563,34 @@ static void build_net(vlog_node_t v, c_abstractScope *scope)
 {
    c_net *net = new_object(sizeof(c_net), vpiNet);
    init_abstractDecl(&(net->decl), v, scope);
+   net->size = vlog_size(vlog_type(v));
 
    vpi_list_add(&scope->decls.list, &(net->decl.object));
 }
 
 static void build_reg(vlog_node_t v, c_abstractScope *scope)
 {
-   c_reg *reg = new_object(sizeof(c_reg), vpiNet);
-   init_abstractDecl(&(reg->decl), v, scope);
+   const int nranges = vlog_ranges(v);
+   if (nranges > 0) {
+      vlog_node_t d0 = vlog_range(v, 0);
+      assert(vlog_subkind(d0) == V_DIM_UNPACKED);
 
-   vpi_list_add(&scope->decls.list, &(reg->decl.object));
+      c_regArray *arr = new_object(sizeof(c_regArray), vpiRegArray);
+      init_abstractDecl(&(arr->decl), v, scope);
+      arr->elems.fn = vpi_lazy_elems;
+      arr->left = build_expr(vlog_left(d0), scope);
+      arr->right = build_expr(vlog_right(d0), scope);
+
+      vpi_list_add(&scope->decls.list, &(arr->decl.object));
+   }
+   else {
+      c_reg *reg = new_object(sizeof(c_reg), vpiReg);
+      init_abstractDecl(&(reg->decl), v, scope);
+      reg->size = vlog_size(vlog_type(v));
+      reg->offset = 0;
+
+      vpi_list_add(&scope->decls.list, &(reg->decl.object));
+   }
 }
 
 static c_constant *build_constant(vlog_node_t v)
@@ -640,6 +701,15 @@ static c_sysFuncCall *build_sysFuncCall(vlog_node_t where, c_callback *callback,
    return call;
 }
 
+static void build_parameter(vlog_node_t v, c_abstractScope *scope)
+{
+   c_parameter *param = new_object(sizeof(c_parameter), vpiParameter);
+   init_abstractDecl(&(param->decl), v, scope);
+   param->value = build_expr(vlog_value(v), scope);
+
+   vpi_list_add(&scope->decls.list, &(param->decl.object));
+}
+
 static bool init_iterator(c_iterator *it, PLI_INT32 type, c_vpiObject *obj)
 {
    c_tfCall *call = is_tfCall(obj);
@@ -708,6 +778,14 @@ static c_abstractScope *cached_scope(tree_t block, rt_scope_t *s)
    return as;
 }
 
+static rt_model_t *vpi_get_model(vpi_context_t *c)
+{
+   if (c->model != NULL)
+      return c->model;
+
+   return get_model();
+}
+
 static jit_t *vpi_get_jit(vpi_context_t *c)
 {
    if (c->jit != NULL)
@@ -737,6 +815,27 @@ static void *vpi_get_ptr(c_abstractDecl *decl)
    return (decl->mptr = jit_get_frame_var(jit, decl->scope->handle, name));
 }
 
+static bool vpi_get_range(c_vpiObject *obj, int64_t *ileft, int64_t *iright)
+{
+   c_constant *left = NULL, *right = NULL;
+
+   c_regArray *arr = is_regArray(obj);
+   if (arr != NULL) {
+      left = is_constantOrParam(arr->left);
+      right = is_constantOrParam(arr->right);
+   }
+
+   if (right == NULL || left == NULL) {
+      vpi_error(vpiError, &(obj->loc), "object has unknown range");
+      return false;
+   }
+
+   *ileft = number_integer(vlog_number(left->expr.where));
+   *iright = number_integer(vlog_number(right->expr.where));
+
+   return true;
+}
+
 static void vpi_lazy_decls(c_vpiObject *obj)
 {
    c_abstractScope *s = is_abstractScope(obj);
@@ -755,12 +854,40 @@ static void vpi_lazy_decls(c_vpiObject *obj)
       case V_VAR_DECL:
          build_reg(v, s);
          break;
+      case V_LOCALPARAM:
+         build_parameter(v, s);
+         break;
       default:
          break;
       }
    }
 }
 
+static void vpi_lazy_elems(c_vpiObject *obj)
+{
+   c_regArray *arr = is_regArray(obj);
+   assert(arr != NULL);
+
+   int64_t left, right;
+   if (!vpi_get_range(obj, &left, &right))
+      return;
+
+   PLI_UINT32 count = left > right ? left - right + 1 : right - left + 1;
+
+   vpi_list_reserve(&arr->elems.list, count);
+
+   PLI_UINT32 size = vlog_size(vlog_type(arr->decl.where));
+
+   for (int64_t i = 0; i < count; i++) {
+      c_reg *reg = new_object(sizeof(c_reg), vpiReg);
+      init_abstractDecl(&(reg->decl), arr->decl.where, arr->decl.scope);
+      reg->size = size;
+      reg->offset = left > right ? (count - 1 - i) * size : i * size;
+
+      vpi_list_add(&arr->elems.list, &(reg->decl.object));
+   }
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // Public API
 
@@ -829,6 +956,46 @@ vpiHandle vpi_handle(PLI_INT32 type, vpiHandle refHandle)
    return NULL;
 }
 
+DLLEXPORT
+vpiHandle vpi_handle_by_name(PLI_BYTE8 *name, vpiHandle scope)
+{
+   VPI_MISSING;
+}
+
+DLLEXPORT
+vpiHandle vpi_handle_by_index(vpiHandle handle, PLI_INT32 index)
+{
+   vpi_clear_error();
+
+   VPI_TRACE("handle=%s index=%d", handle_pp(handle), index);
+
+   c_vpiObject *obj = from_handle(handle);
+   if (obj == NULL)
+      return NULL;
+
+   int64_t left, right;
+   if (!vpi_get_range(obj, &left, &right))
+      return NULL;
+
+   const int64_t low = left < right ? left : right;
+   const int64_t high = left < right ? right : left;
+
+   if (index < low || index > high) {
+      vpi_error(vpiError, &(obj->loc), "index %d out of range", index);
+      return NULL;
+   }
+
+   c_regArray *arr = is_regArray(obj);
+   if (arr != NULL) {
+      vpiObjectList *elems = expand_lazy_list(&arr->decl.object, &arr->elems);
+      assert(index - low < elems->count);
+      return user_handle_for(elems->items[index - low]);
+   }
+
+   vpi_error(vpiError, &(obj->loc), "handle cannot be indexed");
+   return NULL;
+}
+
 DLLEXPORT
 vpiHandle vpi_iterate(PLI_INT32 type, vpiHandle refHandle)
 {
@@ -869,6 +1036,7 @@ vpiHandle vpi_scan(vpiHandle iterator)
    if (it->pos < it->list->count)
       return user_handle_for(it->list->items[it->pos++]);
 
+   drop_handle(vpi_context(), iterator);
    return NULL;
 }
 
@@ -887,7 +1055,7 @@ PLI_INT32 vpi_get(PLI_INT32 property, vpiHandle object)
    if (property == vpiType)
       return obj->type;
 
-   c_constant *con = is_constant(obj);
+   c_constant *con = is_constantOrParam(obj);
    if (con != NULL) {
       switch (property) {
       case vpiConstType:
@@ -904,24 +1072,33 @@ PLI_INT32 vpi_get(PLI_INT32 property, vpiHandle object)
       switch (property) {
       case vpiSize:
          {
+            if (op->subtype == vpiNullOp)
+               return 0;
+
             vpi_context_t *c = vpi_context();
             if (c->args != NULL)
                return c->args[op->argslot].integer;
 
             goto missing_property;
          }
+      case vpiOpType:
+         return op->subtype;
       }
    }
 
-   c_abstractDecl *decl = is_abstractDecl(obj);
-   if (decl != NULL) {
+   c_reg *reg = is_reg(obj);
+   if (reg != NULL) {
       switch (property) {
       case vpiSize:
-         {
-            sig_shared_t **ss = vpi_get_ptr(decl);
-            rt_signal_t *s = container_of(*ss, rt_signal_t, shared);
-            return signal_width(s);
-         }
+         return reg->size;
+      }
+   }
+
+   c_net *net = is_net(obj);
+   if (net != NULL) {
+      switch (property) {
+      case vpiSize:
+         return net->size;
       }
    }
 
@@ -945,7 +1122,7 @@ void vpi_get_value(vpiHandle handle, p_vpi_value value_p)
    vpi_context_t *c = vpi_context();
    tb_rewind(c->valuestr);
 
-   c_constant *con = is_constant(obj);
+   c_constant *con = is_constantOrParam(obj);
    if (con != NULL) {
       switch (con->subtype) {
       case vpiStringConst:
@@ -966,26 +1143,22 @@ void vpi_get_value(vpiHandle handle, p_vpi_value value_p)
             const uint64_t *abits, *bbits;
             number_get(n, &abits, &bbits);
 
-            vpi_format_number(number_width(n), abits, bbits, value_p->format,
+            vpi_format_number(number_width(n), abits, bbits, value_p,
                               c->valuestr);
-
-            value_p->value.str = (PLI_BYTE8 *)tb_get(c->valuestr);
             return;
          }
       }
    }
 
    c_operation *op = is_operation(obj);
-   if (op != NULL && c->args != NULL) {
+   if (op != NULL && c->args != NULL && op->subtype != vpiNullOp) {
       int size = c->args[op->argslot].integer;
       assert(size <= 64);
 
       uint64_t abits[1] = { c->args[op->argslot + 1].integer };
       uint64_t bbits[1] = { c->args[op->argslot + 2].integer };
 
-      vpi_format_number(size, abits, bbits, value_p->format, c->valuestr);
-
-      value_p->value.str = (PLI_BYTE8 *)tb_get(c->valuestr);
+      vpi_format_number(size, abits, bbits, value_p, c->valuestr);
       return;
    }
 
@@ -994,28 +1167,43 @@ void vpi_get_value(vpiHandle handle, p_vpi_value value_p)
       sig_shared_t **ss = vpi_get_ptr(decl);
       rt_signal_t *s = container_of(*ss, rt_signal_t, shared);
 
-      const int width = signal_width(s);
-      const int nwords = (width + 63) / 64;
-      uint64_t *mem LOCAL = xmalloc_array(nwords * 2, sizeof(uint64_t));
-      uint64_t *abits = mem, *bbits = mem + nwords;
+      vlog_node_t type = vlog_get_type(decl->where);
+      if (type == NULL || vlog_kind(type) != V_DATA_TYPE)
+         goto fail;
 
-      for (int i = 0; i < nwords * 2; i++)
-         mem[i] = 0;
+      switch (value_p->format) {
+      case vpiRealVal:
+         if (vlog_subkind(type) != DT_REAL)
+            goto fail;
 
-      const uint8_t *vals = signal_value(s);
+         value_p->format = unaligned_load(signal_value(s), double);
+         return;
 
-      for (int i = 0; i < width; i++) {
-         const int pos = width - 1 - i;
-         abits[pos / 64] |= (uint64_t)(vals[i] & 1) << (pos % 64);
-         bbits[pos / 64] |= (uint64_t)((vals[i] >> 1) & 1) << (pos % 64);
+      default:
+         {
+            const int width = signal_width(s);
+            const int nwords = (width + 63) / 64;
+            uint64_t *mem LOCAL = xmalloc_array(nwords * 2, sizeof(uint64_t));
+            uint64_t *abits = mem, *bbits = mem + nwords;
+
+            for (int i = 0; i < nwords * 2; i++)
+               mem[i] = 0;
+
+            const uint8_t *vals = signal_value(s);
+
+            for (int i = 0; i < width; i++) {
+               const int pos = width - 1 - i;
+               abits[pos / 64] |= (uint64_t)(vals[i] & 1) << (pos % 64);
+               bbits[pos / 64] |= (uint64_t)((vals[i] >> 1) & 1) << (pos % 64);
+            }
+
+            vpi_format_number(width, abits, bbits, value_p, c->valuestr);
+            return;
+         }
       }
-
-      vpi_format_number(width, abits, bbits, value_p->format, c->valuestr);
-
-      value_p->value.str = (PLI_BYTE8 *)tb_get(c->valuestr);
-      return;
    }
 
+ fail:
    vpi_error(vpiError, &(obj->loc), "cannot evaluate %s", handle_pp(handle));
 }
 
@@ -1051,7 +1239,8 @@ vpiHandle vpi_put_value(vpiHandle handle, p_vpi_value value_p,
          {
             assert(value_p->format == vpiIntVal);
 
-            c->args[0].integer = value_p->value.integer;
+            // TODO: mask should be based on result size
+            c->args[0].integer = value_p->value.integer & 0xffffffff;
             c->args[1].integer = 0;
 
             return NULL;
@@ -1059,6 +1248,48 @@ vpiHandle vpi_put_value(vpiHandle handle, p_vpi_value value_p,
       }
    }
 
+   c_reg *reg = is_reg(obj);
+   if (reg != NULL) {
+      sig_shared_t **ss = vpi_get_ptr(&reg->decl);
+      rt_signal_t *s = container_of(*ss, rt_signal_t, shared);
+
+      uint8_t *unpacked LOCAL = xcalloc_array(reg->size, sizeof(uint8_t));
+
+      switch (value_p->format) {
+      case vpiIntVal:
+         for (int i = 0; i < reg->size && i < 32; i++)
+            unpacked[reg->size - 1 - i] =
+               !!(value_p->value.integer & (1u << i));
+         break;
+
+      case vpiHexStrVal:
+         for (int i = 0; i < reg->size && value_p->value.str[i]; i++) {
+            uint8_t nibble;
+            switch (value_p->value.str[i]) {
+            case '0'...'9': nibble = value_p->value.str[i] - '0'; break;
+            case 'a'...'f': nibble = 10 + value_p->value.str[i] - 'a'; break;
+            case 'A'...'F': nibble = 10 + value_p->value.str[i] - 'A'; break;
+            default:
+               vpi_error(vpiError, NULL, "invalid character %c in hex string",
+                         value_p->value.str[i]);
+               return NULL;
+            }
+
+            for (int j = i * 4; j < reg->size && j < i * 4 + 4; j++)
+               unpacked[j] = !!(nibble & (1 << (3 - j + i * 4)));
+         }
+         break;
+
+      default:
+         vpi_error(vpiError, &(obj->loc), "unsupported format %d",
+                   value_p->format);
+         return NULL;
+      }
+
+      deposit_signal(vpi_get_model(c), s, unpacked, reg->offset, reg->size);
+      return NULL;
+   }
+
    vpi_error(vpiError, &(obj->loc), "cannot change value of %s",
              handle_pp(handle));
    return NULL;
