@@ -55,7 +55,13 @@
 #define realpath(N, R) _fullpath((R), (N), _MAX_PATH)
 #else
 #include <sys/wait.h>
+#include <glob.h>
 #endif
+
+#ifdef __linux__
+#include <sys/personality.h>
+#endif
+
 #include "config.h"
 
 #define WHITESPACE " \t\r\n"
@@ -154,12 +160,12 @@ static test_t *test_list = NULL;
 static char test_dir[PATH_MAX];
 static char bin_dir[PATH_MAX];
 static bool is_tty = false;
-static bool force_jit = false;
-static bool force_precompile = false;
 static bool capture_xml = false;
 static bool capture_html = false;
 static bool diff_html = false;
 static bool check_html = false;
+static bool capture_llvm = false;
+static bool diff_llvm = false;
 
 #ifdef __MINGW32__
 static char *strndup(const char *s, size_t n)
@@ -611,6 +617,13 @@ static run_status_t run_cmd(FILE *log, arglist_t **args)
       if (nullfd != -1)
          dup2(nullfd, STDIN_FILENO);
 
+#ifdef __linux__
+      if ((capture_llvm || diff_llvm) && personality(ADDR_NO_RANDOMIZE) == -1) {
+         fprintf(stderr, "Failed to disable ASLR: %s\n", strerror(errno));
+         _exit(EXIT_FAILURE);
+      }
+#endif
+
       char **argv = calloc((*args)->count + 1, sizeof(char *));
       arglist_t *it = *args;
       for (int i = 0; i < (*args)->count; i++, it = it->next) {
@@ -960,6 +973,10 @@ static bool run_test(test_t *test)
             push_arg(&args, "--cover=%s", test->cover);
          else
             push_arg(&args, "--cover");
+
+         if (file_exists("%s/regress/data/%s.spec", test_dir, test->name))
+            push_arg(&args, "--cover-spec=%s/regress/data/%s.spec",
+                     test_dir, test->name);
       }
 
       for (param_t *p = test->params; p != NULL; p = p->next) {
@@ -973,11 +990,6 @@ static bool run_test(test_t *test)
          }
       }
 
-      if (force_jit)
-         push_arg(&args, "--jit");
-      else if (force_precompile)
-         push_arg(&args, "--precompile");
-
       if (test->flags & F_FAIL) {
          if (run_cmd(outf, &args) != RUN_OK) {
             failed(NULL);
@@ -1212,6 +1224,59 @@ static bool run_test(test_t *test)
       }
    }
 
+#ifndef __MINGW32__
+   if ((capture_llvm || diff_llvm) && !(test->flags & (F_SHELL | F_TCL))) {
+      glob_t globbuf = {};
+      if (glob("*.ll", GLOB_DOOFFS, NULL, &globbuf) != 0) {
+         failed("failed to glob LLVM output");
+         result = false;
+         goto out_close;
+      }
+
+      push_arg(&args, "/bin/mv");
+
+      for (int i = 0; i < globbuf.gl_pathc; i++)
+         push_arg(&args, "%s", globbuf.gl_pathv[i]);
+
+      char *llvm_dir = xasprintf("%s/llvm/%s", cwd, test->name);
+
+      if (diff_llvm) {
+         make_dir("llvm");
+         push_arg(&args, "llvm");
+      }
+      else
+         push_arg(&args, "%s", llvm_dir);
+
+      if (!dir_exists(llvm_dir) && !make_dir(llvm_dir)) {
+         failed("create LLVM output dir");
+         result = false;
+         goto out_close;
+      }
+      else if (run_cmd(outf, &args) != RUN_OK) {
+         failed("move LLVM output");
+         result = false;
+         goto out_print;
+      }
+
+      if (diff_llvm) {
+         push_arg(&args, DIFF_PATH);
+         push_arg(&args, "-u");
+         push_arg(&args, "--color=always");
+
+         push_arg(&args, "%s", llvm_dir);
+         push_arg(&args, "llvm");
+
+         if (run_cmd(outf, &args) != RUN_OK) {
+            failed("LLVM output differs");
+            result = false;
+            goto out_print;
+         }
+      }
+
+      free(llvm_dir);
+   }
+#endif
+
    if (test->flags & F_GTKW) {
       push_arg(&args, "%s", DIFF_PATH);
 #if defined __MINGW32__ || defined __CYGWIN__
@@ -1404,23 +1469,19 @@ int main(int argc, char **argv)
       { "diff-html",    no_argument, 0, 'H' },
       { "check-html",   no_argument, 0, 'C' },
       { "capture-xml",  no_argument, 0, 'x' },
+      { "capture-llvm", no_argument, 0, 'l' },
+      { "diff-llvm",    no_argument, 0, 'L' },
       { 0, 0, 0, 0 }
    };
 
    bool print_stats= false;
    int c, index = 0;
-   const char *spec = ":sjp";
+   const char *spec = ":s";
    while ((c = getopt_long(argc, argv, spec, long_options, &index)) != -1) {
       switch (c) {
       case 's':
          print_stats = true;
          break;
-      case 'j':
-         force_jit = true;
-         break;
-      case 'p':
-         force_precompile = true;
-         break;
       case 'h':
          capture_html = true;
          make_dir("html");
@@ -1434,6 +1495,13 @@ int main(int argc, char **argv)
       case 'C':
          check_html = true;
          break;
+      case 'l':
+         capture_llvm = true;
+         make_dir("llvm");
+         break;
+      case 'L':
+         diff_llvm = true;
+         break;
       case '?':
          if (optopt == 0)
             fprintf(stderr, "Invalid option %s\n", argv[optind - 1]);
@@ -1455,6 +1523,12 @@ int main(int argc, char **argv)
    setenv("TESTDIR", test_dir, 1);
 #endif
 
+   if (capture_llvm || diff_llvm) {
+      setenv("NVC_LLVM_VERBOSE", "1", 1);
+      setenv("NVC_JIT_THRESHOLD", "1", 1);
+      setenv("NVC_JIT_ASYNC", "0", 1);
+   }
+
    free(newpath);
 
    unsigned mask = 0;
