@@ -1,5 +1,5 @@
 //
-//  Copyright (C) 2011-2025  Nick Gasson
+//  Copyright (C) 2011-2026  Nick Gasson
 //
 //  This program is free software: you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -2270,6 +2270,21 @@ START_TEST(test_clone3)
 }
 END_TEST
 
+START_TEST(test_issue1421)
+{
+   opt_set_int(OPT_PRESERVE_CASE, 1);
+
+   elab_set_generic("MY_GENERIC", "5");
+
+   input_from_file(TESTDIR "/elab/issue1421.vhd");
+
+   tree_t e = run_elab();
+   fail_if(e == NULL);
+
+   fail_if_errors();
+}
+END_TEST
+
 Suite *get_elab_tests(void)
 {
    Suite *s = suite_create("elab");
@@ -2385,6 +2400,7 @@ Suite *get_elab_tests(void)
    tcase_add_test(tc, test_issue1333);
    tcase_add_test(tc, test_issue1404);
    tcase_add_test(tc, test_clone3);
+   tcase_add_test(tc, test_issue1421);
    suite_add_tcase(s, tc);
 
    return s;
