@@ -48,7 +48,7 @@ START_TEST(test_examine1)
 
    mir_context_t *mc = get_mir();
    unit_registry_t *ur = get_registry();
-   jit_t *j = jit_new(ur, mc, NULL);
+   jit_t *j = jit_new(ur, mc);
 
    input_from_file(TESTDIR "/shell/examine1.vhd");
 
@@ -194,7 +194,7 @@ START_TEST(test_wave1)
 {
    mir_context_t *mc = get_mir();
    unit_registry_t *ur = get_registry();
-   jit_t *j = jit_new(ur, mc, NULL);
+   jit_t *j = jit_new(ur, mc);
 
    tcl_shell_t *sh = shell_new(j);
 
@@ -338,7 +338,9 @@ END_TEST
 static void force1_stdout_handler(const char *buf, size_t nchars, void *user)
 {
    static const char *expect[] = {
-      "force /x '1'\n", "force /y 42\n", "force /z \"110\"\n"
+      "force ", "/x", " ", "'1'", "\n",
+      "force ", "/y", " ", "42", "\n",
+      "force ", "/z", " ", "\"110\"", "\n"
    };
 
    int *state = user;
@@ -363,7 +365,7 @@ START_TEST(test_force1)
 
    mir_context_t *mc = get_mir();
    unit_registry_t *ur = get_registry();
-   jit_t *j = jit_new(ur, mc, NULL);
+   jit_t *j = jit_new(ur, mc);
 
    tree_t arch = parse_check_and_simplify(T_ENTITY, T_ARCH);
 
@@ -398,7 +400,7 @@ START_TEST(test_force1)
 
    shell_eval(sh, "force", &result);
    ck_assert_str_eq(result, "");
-   ck_assert_int_eq(state, 3);
+   ck_assert_int_eq(state, 15);
 
    shell_eval(sh, "run 1 ns", &result);
    ck_assert_str_eq(result, "");
@@ -467,7 +469,7 @@ START_TEST(test_describe1)
 
    mir_context_t *mc = get_mir();
    unit_registry_t *ur = get_registry();
-   jit_t *j = jit_new(ur, mc, NULL);
+   jit_t *j = jit_new(ur, mc);
 
    input_from_file(TESTDIR "/shell/describe1.vhd");
 
